#!/usr/bin/perl

# several alternative specifications for scaling are possible (they are mutually exclusive)
# mass may be provided either by [*1a] or [*1b] option, length/velocity - by either of [*2...] options
foreach $arg (@ARGV)    # list of possible command-line arguments
{
  $used=0;
  if($arg=~/tab=(.+)/)                { $used=1; $tab=$1;    }  # name of tab file from 'mkspher' (smooth-potentail properties) [no default]
  if($arg=~/mass=([\d\.\+\-de]+)/i)   { $used=1; $Mtot=$1;  }   # total mass of the model in solar masses [*1a]
  if($arg=~/mbh=([\d\.\+\-de]+)/i)    { $used=1; $Mbh=$1;  }    # mass of the central black hole (if present) in solar masses [*1b]
  if($arg=~/rscale=([\d\.\+\-de]+)/i) { $used=1; $rscale=$1; }  # scale radius of the model in parsecs [*2a]
  if($arg=~/rinfl=([\d\.\+\-de]+)/i)  { $used=1; $rinfl=$1; }   # influence radius of the black hole (if present) in parsecs [*2b]
  if($arg=~/r1=([\d\.\+\-de]+)/i)     { $used=1; $r1=$1; }      # some characteristic radius for the density normalization in parsecs [*2c]
  if($arg=~/rho1=([\d\.\+\-de]+)/i)   { $used=1; $rho1=$1; }    # density at the characteristic radius r1 in Msolar/pc^3 [*2c]
  if($arg=~/sigma=([\d\.\+\-de]+)/i)  { $used=1; $sigma=$1; }   # l.o.s. velocity dispersion averaged over half-light radius, in km/s [*2d]
  if($arg=~/vscale=([\d\.\+\-de]+)/i) { $used=1; $vscale=$1; }  # scale velocity of the model in km/s [*2e]
  if(!$used) { die "Parameter $arg unknown"; }
}

# convenience functions
sub pow   # (x, y) => x^y
{
  my $x=shift;
  my $p=shift;
  return exp($p*log($x));
}
sub pow2  # (x) => x^2, works also for non-positive x
{
  my $x=shift;
  return $x*$x;
}
sub interp  # (x, x1, x2, y1, y2) => give linearly interpolated y(x)
{
  my $x=shift;
  my $x1=shift;
  my $x2=shift;
  my $y1=shift;
  my $y2=shift;
  return $y1+($x-$x1)*($y2-$y1)/($x2-$x1);
}
sub integr1  # (x1, x2, y1, y2) => integrate 2*pi*x*interp(x, x1, x2, y1, y2)
{
  my $x1=shift;
  my $x2=shift;
  my $y1=shift;
  my $y2=shift;
  if($x1==0) { $y1=$y2; }  # ignore value at x1=0
  return ($x2-$x1)*((2*$y1+$y2)*$x1+($y1+2*$y2)*$x2)*3.141593/3;
}

if(!$tab)    { die "Must specify input table: tab=???"; }

open FI, "<$tab" or die("Can't open input file $tab");
$numlines=0;
while(<FI>)
{
  s/^\s+//;
  chomp;
  if(/^\d/) {   # if line starts with a digit
    $lines[$numlines]=$_;
    $numlines++;
    @L=split(/\s+/);
    if($L[0]==0) { $tildeMbh=$L[1]; }
    $tildeMtot=$L[1];
  }
}
close FI;

if(!$Mtot && !$Mbh || $Mbh && $Mtot)
  { die "Either total mass (Mtot=???) or black hole mass (Mbh=???) must be provided (but not both)"; }
if($Mbh && !$tildeMbh) 
  { die "No central black hole is found in tab file"; }
# mass scale: how many solar masses in one mass unit in the tab file
if($Mtot) { $mscale=$Mtot/$tildeMtot; }
if($Mbh)  { $mscale=$Mbh/$tildeMbh; $Mtot=$tildeMtot*$mscale; }

# length, density or velocity scale
$mode=0;
$extramsg="Must provide one and only one of the options: rscale=???, rinfl=???, {r1=??? and rho1=???}, vscale=???, sigma=???";
if($rscale) { $mode=1; }
if($rinfl) { 
  if($mode) { die $extramsg; }
  if(!$tildeMbh) { die "No black hole in tab file -- no influence radius..."; }
  $mode=2;
}
if($r1 && $rho1) {
  if($mode) { die $extramsg; }
  $mode=3;
}
if($vscale) {
  if($mode) { die $extramsg; }
  $mode=4;
}
if($sigma) {
  if($mode) { die $extramsg; }
  $mode=5;
}
if($mode==0) { die $extramsg; }

#if($mode==2) {
{ # compute the influence and half-mass/half-light radii anyway
  $prevm=$tildeMbh;
  $prevr=0;
  foreach $line(@lines) {
    @L=split(/\s+/, $line);
    if($L[1]>=3*$tildeMbh && $prevm<3*$tildeMbh) { # in between the two values
      $tilderinfl = interp(3*$tildeMbh, $prevm, $L[1], $prevr, $L[0]);
    }
    if($L[1]>=0.5*$tildeMtot && $prevm<0.5*$tildeMtot) {
      $tilderhm = interp(0.5*$tildeMtot, $prevm, $L[1], $prevr, $L[0]);
    }
    $intlight += integr1($prevr, $L[0], $prevlight, $L[12]);
    $intlosvd += integr1($prevr, $L[0], $prevlosvd, $L[12]*$L[11]);
    if($intlight>=0.5*$tildeMtot && $previntlight<0.5*$tildeMtot) {
      $tilderhl = interp(0.5*$tildeMtot, $previntlight, $intlight, $prevr, $L[0]);
      $tildesigmaavg = $intlosvd/$intlight;   # no interpolation, as it is not expected to change rapidly
      $tildesigmarhl = $L[11];  # same remark
    }
    $prevr=$L[0];
    $prevm=$L[1];
    $prevlight=$L[12];
    $prevlosvd=$L[11]*$L[12];
    $previntlight=$intlight;
  }
  if($mode==2) {  # this value is used to scale the model length unit
    if(!$tilderinfl) { die "Cannot compute influence radius from tab file (black hole is too massive?)"; }
    $rscale=$rinfl/$tilderinfl;
  }
}

if($mode==3) {
  $tilder3rho_req = $rho1*pow($r1,3)/$mscale; 
  $prevr3rho=0;
  $prevr=0;
  foreach $line(@lines) {
    @L=split(/\s+/, $line);
    $tilder3rho=$L[3]*$L[0]*pow2($L[0]);
    if($tilder3rho>=$tilder3rho_req && $prevr3rho<$tilder3rho_req) { # in between the two values
      $tilder1 = interp($tilder3rho_req, $prevr3rho, $tilder3rho, $prevr, $L[0]);
    }
    $prevr=$L[0];
    $prevr3rho=$tilder3rho;
  }
  if(!$tilder1) { die "Cannot find an appropriate characteristic radius-density pair in tab file"; }
  $rscale=$r1/$tilder1;
}

if($mode==5) { 
  if(!$tildesigmaavg) { die "Cannot compute effective velocity dispersion at r_halflight from tab file"; }
  $vscale=$sigma/$tildesigmaavg;
}

if(!$vscale) {
  $vscale = 0.0658*sqrt($mscale/$rscale);
} else {
  $rscale = $mscale*pow2(0.0658/$vscale);
}
$tscale = 0.98e6*$rscale/$vscale;
#$tscale1=1.48e7*pow($rscale,1.5)/sqrt($mscale);  # model time unit in years
print "mscale=$mscale M_sun, rscale=$rscale pc, vscale=$vscale km/s, tscale=$tscale yr\n";
print "r_infl=".($tilderinfl*$rscale)." pc, r_halfmass=".($tilderhm*$rscale)." pc, ".
      "r_halflight=".($tilderhl*$rscale)." pc, sigma_1d(r_hl)=".($tildesigmarhl*$vscale).", sigma_avg=".($tildesigmaavg*$vscale)." km/s\n";
