/** \file    utils.h
    \brief   several mathematical, statistical, string formatting and error reporting routines
    \author  EV
    \date    2009-2013

This file is a shortened version of the eponimous file from SMILE toolbox.

*/
#pragma once
#include <string>
#include <sstream>
#include <iomanip>
#include <vector>
#include <gsl/gsl_math.h>
#include <gsl/gsl_spline.h>

namespace smile{

/*------------- string functions ------------*/
/** The StringVariant class is a simple string-based variant implementation that allows
    the user to easily convert between simple numeric/string types.
*/
class StringVariant
{
private:
    std::string data;
public:
    StringVariant() : data() {}
    StringVariant(const std::string &src) : data(src) {};
    template<typename ValueType> StringVariant(ValueType val) {
        std::ostringstream stream;
        stream << val;
        data.assign(stream.str());
    };
    template<typename ValueType> StringVariant(ValueType val, unsigned int width) {
        std::ostringstream stream;
        stream << std::setprecision(width) << val;
        data.assign(stream.str());
    };
    template<typename ValueType> StringVariant& operator=(const ValueType val) {
        std::ostringstream stream;
        stream << val;
        data.assign(stream.str());
        return *this;
    };
    template<typename NumberType> NumberType toNumber() const {
        NumberType result = 0;
        std::istringstream stream(data);
        if(stream >> result)
            return result;
        else if(data == "yes" || data == "true")
            return 1;
        return 0;
    };
    bool toBool() const { return(data == "yes" || data == "true" || data == "t" || data == "1"); }
    double toDouble() const { return toNumber<double>(); }
    float toFloat() const { return toNumber<float>(); }
    int toInt() const { return toNumber<int>(); }
    std::string toString() const { return data; }
};

/// a shorthand function to convert a string to a number
template<typename NumberType, typename ValueType> NumberType convertTo(ValueType val) 
  { return StringVariant(val).toNumber<NumberType>(); }

/// a shorthand function to convert a number to a string
template<typename ValueType> std::string convertToString(ValueType val) 
  { return StringVariant(val).toString(); }

/// a shorthand function to convert a number to a string with a given precision
template<typename ValueType> std::string convertToString(ValueType val, unsigned int width) 
  { return StringVariant(val,width).toString(); }

/*---------- math functions ------------*/
/** generates a grid with exponentially growing spacing. 
    x[k] = (exp(Z k) - 1)/(exp(Z) - 1), 
    and the value of Z is computed so the the 1st element is at xmin and last at xmax; 
    if zeroelem=true, 0th node is at zero (otherwise at xmin). 
*/
void createNonuniformGrid(size_t nnodes, double xmin, double xmax, bool zeroelem, std::vector<double> *grid);

/*---------- convenience function for numerically solving an equation ------*/
/** wrapper for GSL root solver running multiple iterations until the convergence is achieved.
    may additionally specify desired relative tolerance and default value to be returned 
    if endpoints don't enclose root. 
*/
double findRoot(gsl_function *F, double xlower, double xupper, double xdefault=0, double reltoler=1e-8);

/** wrapper for GSL minimization routine to find local minimum enclosed between xlower < xinit < xupper.
    xinit is the initial guess and the value to be returned in the case of error. 
*/
double findMin(gsl_function *F, double xlower, double xupper, double xinit, double reltoler=1e-8);

/** wrapper for GSL minimization routine to find local minimum enclosed between xlower < xupper.
    employed when the initial minimum location is not really known but suspected to exist in the interval.
    performs binary search to find the initial minimum location and then calls GSL routines to refine it.
    if the interval doesn't enclose minimum then one of the endpoints is returned. 
*/
double findMinGuess(gsl_function *F, double xlower, double xupper, double reltoler=1e-8);

/*---------- spline functions -----------*/
/** a variant of cubic spline with given values of first derivative at endpoints.
    these values are passed as two additional points in y[i] array 
    ( y[npoints] for leftmost, y[npoints+1] for rightmost).
    if one wants to leave the standard, natural boundary condition (second derivative is zero),
    then pass NaN as a correspoinding 1st derivative parameter. 
*/
extern const gsl_interp_type * gsl_interp_ccspline;

/// modified functions for spline evaluation that extrapolate last segment linearly beyond the definition region
double gsl_spline_evalx(const gsl_spline *spline, double x, gsl_interp_accel *a );
double gsl_spline_eval_derivx(const gsl_spline *spline, double x, gsl_interp_accel *a );
double gsl_spline_eval_deriv2x(const gsl_spline *spline, double x, gsl_interp_accel *a );

}  // namespace